use clap::{Parser, Subcommand};

#[derive(Debug, Parser)]
#[clap(author, version, about, long_about = None)]
struct RootArgs {
    #[clap(default_value_t = String::from("vogt.db"))]
    db: String,

    #[clap(short, long, default_value_t = String::from("primary"))]
    realm: String,

    #[clap(subcommand)]
    command: Command,
}

#[derive(Debug, Subcommand)]
enum Command {
    Init,
    Cert(CertArgs),
    Server(ServerArgs),
}

impl RootArgs {
    async fn run(&self) {
        if let Command::Init = self.command {
            return self.init().await;
        }

        let storage = microrm::DB::new(
            crate::schema::schema(),
            &self.db,
            microrm::CreateMode::MustExist,
        );

        if let Err(e) = storage {
            println!("Error occured while loading database: {}", e);
            return;
        }
        let storage = storage.unwrap();

        match &self.command {
            Command::Init => unreachable!(),
            Command::Cert(v) => v.run(&self, &storage).await,
            Command::Server(v) => v.run(&self, storage).await,
        }
    }

    async fn init(&self) {
        // first check to see if the database is already vaguely set up
        let maybedb = microrm::DB::new(
            crate::schema::schema(),
            &self.db,
            microrm::CreateMode::MustExist,
        );

        if maybedb.is_ok() {
            println!("Database already initialized, not overwriting!");
            return;
        }

        println!("Initializing!");

        let db = microrm::DB::new(
            crate::schema::schema(),
            &self.db,
            microrm::CreateMode::AllowNewDatabase,
        )
        .expect("Unable to initialize database!");

        // create primary realm
        db.query_interface().add(&crate::schema::Realm {
            shortname: "primary".to_string(),
        });
    }
}

#[derive(Debug, Subcommand)]
enum CertCommand {
    Inspect,
    Generate,
}

#[derive(Debug, Parser)]
struct CertArgs {
    #[clap(subcommand)]
    command: CertCommand,
}

impl CertArgs {
    async fn run(&self, root: &RootArgs, si: &microrm::DB) {
        match &self.command {
            CertCommand::Inspect => {
                crate::cert::inspect(si, &root.realm);
            }
            CertCommand::Generate => {
                crate::cert::generate(si, &root.realm);
            }
        }
    }
}

#[derive(Debug, Parser)]
struct ServerArgs {}

impl ServerArgs {
    async fn run(&self, root: &RootArgs, db: microrm::DB) {
        crate::server::run_server(db).await;
    }
}

pub fn invoked() {
    let args = RootArgs::parse();

    async_std::task::block_on(args.run());
}
